"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
var platform_1 = require("tns-core-modules/platform");
var types_1 = require("utils/types");
var index_common_1 = require("./index.common");
var observable_1 = require("tns-core-modules/data/observable");
__export(require("./index.common"));
function notifyForItemAtIndex(owner, nativeView, view, eventName, index) {
    var args = {
        eventName: eventName,
        object: owner,
        index: index,
        view: view,
        ios: nativeView,
        android: undefined
    };
    owner.notify(args);
    return args;
}
var MDCSwipeDelegateImpl = (function (_super) {
    __extends(MDCSwipeDelegateImpl, _super);
    function MDCSwipeDelegateImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MDCSwipeDelegateImpl.initWithOwner = function (owner) {
        var delegate = MDCSwipeDelegateImpl.alloc().init();
        delegate._owner = owner;
        return delegate;
    };
    MDCSwipeDelegateImpl.prototype.viewDidCancelSwipe = function (view) {
        this._owner.get()._onSwipeCancelled();
    };
    MDCSwipeDelegateImpl.prototype.viewWasChosenWithDirection = function (view, direction) {
        this._owner.get()._onSwipe(view, direction);
    };
    MDCSwipeDelegateImpl.ObjCProtocols = [MDCSwipeToChooseDelegate];
    return MDCSwipeDelegateImpl;
}(NSObject));
var CardStack = (function (_super) {
    __extends(CardStack, _super);
    function CardStack() {
        var _this = _super.call(this) || this;
        _this._viewMap = new Map();
        _this._visibleArray = [];
        _this._delegate = MDCSwipeDelegateImpl.initWithOwner(new WeakRef(_this));
        return _this;
    }
    Object.defineProperty(CardStack.prototype, "ios", {
        get: function () {
            return this._ios;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CardStack.prototype, "topPosition", {
        get: function () {
            return this.selectedIndex;
        },
        enumerable: true,
        configurable: true
    });
    CardStack.prototype.createNativeView = function () {
        if (this.options && this.visibleCount) {
            this.visibleCount = this.options.visibleCount;
        }
        this._ios = MDCSwipeToChooseView.alloc().initWithFrame(CGRectMake(0, 0, platform_1.screen.mainScreen.widthDIPs, platform_1.screen.mainScreen.heightDIPs));
        return this._ios;
    };
    CardStack.prototype.initNativeView = function () {
    };
    CardStack.prototype.disposeNativeView = function () {
        this.removeChildren();
    };
    CardStack.prototype.onLoaded = function () {
        _super.prototype.onLoaded.call(this);
        this.refresh();
        this.notify({
            eventName: index_common_1.CardStackCommon.loadedEvent,
            object: this
        });
    };
    CardStack.prototype._onSwipe = function (view, direction) {
        this.notify({
            eventName: index_common_1.CardStackCommon.swipedEvent,
            object: observable_1.fromObject({
                direction: direction === 2 ? 'Right' : 'Left',
                position: this.topPosition
            })
        });
        this.selectedIndex++;
        this.updateStackAfterSwipe();
    };
    CardStack.prototype._onSwipeCancelled = function () {
        this.notify({
            eventName: index_common_1.CardStackCommon.canceledEvent,
            object: observable_1.fromObject({
                position: this.selectedIndex
            })
        });
    };
    CardStack.prototype.refresh = function (event) {
        if (event === void 0) { event = null; }
        if (event) {
            if (event.action === 'add') {
                if (types_1.isNullOrUndefined(this.items) || !types_1.isNumber(this.items.length)) {
                    return;
                }
                if (!this._ios) {
                    return;
                }
                if (this._viewMap.size < this.items.length) {
                    var cardsInVisibleStack = this._viewMap.size - this.selectedIndex;
                    var toAddCount = this.visibleCount - cardsInVisibleStack;
                    var n = 0;
                    for (var i = this.selectedIndex + cardsInVisibleStack; i < this.selectedIndex + cardsInVisibleStack + toAddCount; i++) {
                        if (this.items.getItem(i)) {
                            this._createCard(this.items.getItem(i), cardsInVisibleStack + n, i);
                            n++;
                        }
                    }
                }
            }
        }
        else {
            if (types_1.isNullOrUndefined(this.items) || !types_1.isNumber(this.items.length)) {
                return;
            }
            if (!this._ios) {
                return;
            }
            this.resetCards();
        }
        return;
    };
    CardStack.prototype.next = function (direction) {
        if (direction === void 0) { direction = 'RIGHT'; }
        if (this._ios) {
            var item = this._viewMap.get(this.selectedIndex);
            if (!item) {
                return;
            }
            if (direction === 'RIGHT') {
                item.mdc_swipe(2);
            }
            else {
                item.mdc_swipe(1);
            }
        }
    };
    CardStack.prototype.previous = function () {
        if (this.selectedIndex === 0) {
            return;
        }
        this.selectedIndex = this.selectedIndex - 1;
        this._viewMap.forEach(function (card, index) {
            card.removeFromSuperview();
        });
        for (var i = this.selectedIndex; i < this.selectedIndex + this.visibleCount; i++) {
            var item = this.items.getItem(i);
            if (item) {
                this._createCard(item, i - this.selectedIndex, i);
            }
        }
        this.notify({
            eventName: index_common_1.CardStackCommon.rewoundEvent,
            object: observable_1.fromObject({
                position: this.selectedIndex
            })
        });
    };
    CardStack.prototype.updateStackAfterSwipe = function () {
        if (this.selectedIndex + this.visibleCount <= this.items.length) {
            this._createCard(this.items.getItem(this.selectedIndex + this.visibleCount - 1), this.visibleCount - 1, this.selectedIndex + this.visibleCount - 1);
        }
        for (var i = this.selectedIndex + 1; i < (this.selectedIndex + this.visibleCount); i++) {
            var cardView = this._viewMap.get(i);
            if (!cardView) {
                return;
            }
        }
    };
    CardStack.prototype._onCardPan = function (state) {
        if (state.direction === 1 || state.direction === 2) {
            this.notify({
                eventName: index_common_1.CardStackCommon.draggingEvent,
                object: observable_1.fromObject({
                    side: state.direction === 1 ? 'Left' : 'Right',
                    ratio: state.thresholdRatio
                })
            });
        }
        for (var i = this.selectedIndex + 1; i <= (this.selectedIndex + this.visibleCount); i++) {
            var cardView = this._viewMap.get(i);
            if (!cardView) {
                return;
            }
            cardView.updatePosition(state.thresholdRatio, i - this.selectedIndex);
        }
    };
    CardStack.prototype.resetCards = function () {
        var _this = this;
        if (types_1.isNullOrUndefined(this.items) || !types_1.isNumber(this.items.length)) {
            return;
        }
        if (this.selectedIndex >= this.items.length - 1) {
            return;
        }
        this.items.forEach(function (item, i) {
            if (i < _this.visibleCount) {
                var cardView = _this._createCard(item, i, i);
            }
        });
    };
    CardStack.prototype._createCard = function (item, visibleStackPosition, index) {
        var cgRect = CGRectMake(0, 0, CGRectGetWidth(this._ios.frame), CGRectGetHeight(this._ios.frame));
        var personViewHolder = ChoosePersonView.initWithOwnerFrameItemPositionDelegate(new WeakRef(this), cgRect, item, visibleStackPosition, this._delegate);
        if (index === 0) {
            this._ios.addSubview(personViewHolder);
        }
        else {
            this._ios.insertSubviewBelowSubview(personViewHolder, this._viewMap.get(index - 1));
        }
        var _args = notifyForItemAtIndex(this, personViewHolder.templateView ? personViewHolder.templateView.nativeView : null, personViewHolder.templateView, index_common_1.ITEMLOADING, index);
        this._viewMap.set(index, personViewHolder);
        return personViewHolder;
    };
    CardStack.prototype.onItemsChanged = function (data) {
        if (!types_1.isNullOrUndefined(this.items) && types_1.isNumber(this.items.length)) {
            this.refresh();
        }
    };
    CardStack.prototype._getDataItem = function (index) {
        return this.items.getItem ? this.items.getItem(index) : this.items[index];
    };
    return CardStack;
}(index_common_1.CardStackCommon));
exports.CardStack = CardStack;
var ChoosePersonView = (function (_super) {
    __extends(ChoosePersonView, _super);
    function ChoosePersonView() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.cardOffset = 10;
        return _this;
    }
    ChoosePersonView.initWithOwnerFrameItemPositionDelegate = function (owner, frame, item, position, delegate) {
        var cardOffset = 5;
        var options = MDCSwipeToChooseViewOptions.alloc().init();
        options.delegate = delegate;
        options.onPan = (function (state) {
            owner.get()._onCardPan(state);
        });
        var yOffset = ((position * cardOffset));
        var viewFrame = CGRectMake(frame.origin.x, frame.origin.y, frame.size.width, frame.size.height);
        var view = ChoosePersonView.alloc().initWithFrameOptions(viewFrame, options);
        view.item = item;
        view.frontViewFrame = CGRectMake(frame.origin.x, frame.origin.y, CGRectGetWidth(frame), CGRectGetHeight(frame));
        view.likedView.hidden = true;
        view.nopeView.hidden = true;
        view.owner = owner;
        view.cardOffset = cardOffset;
        view.layer.borderWidth = 0;
        view.templateView = owner.get()._createTemplateView();
        view.templateView.bindingContext = item;
        view.addSubview(view.templateView.nativeView);
        var topCenter = CGPointMake(CGRectGetMidX(viewFrame), CGRectGetMinY(viewFrame));
        view.layer.anchorPoint = CGPointMake(0.5, 0);
        view.layer.position = topCenter;
        view.transform = CGAffineTransformMakeScale(1 - position / 20, 1 - position / 20);
        view.center = CGPointMake(frame.size.width / 2, -yOffset);
        return view;
    };
    ChoosePersonView.prototype.setFrameBasedOnPosition = function (position) {
        ChoosePersonView.animateWithDurationAnimations(0.2, function () {
        });
    };
    ChoosePersonView.prototype.updatePosition = function (ratio, position) {
        var _this = this;
        var topCenter = CGPointMake(CGRectGetMidX(this.frame), CGRectGetMinY(this.frame));
        this.layer.anchorPoint = CGPointMake(0.5, 0);
        this.layer.position = topCenter;
        var currentScale = 1 - position / 20;
        var nextScale = 1 - ((position - 1) / 20);
        var scale = currentScale + (ratio * (nextScale - currentScale));
        this.transform = CGAffineTransformMakeScale(scale, scale);
        if (ratio < 1) {
            this.center = CGPointMake(this.frontViewFrame.size.width / 2, -(this.cardOffset * position) + (ratio * this.cardOffset));
        }
        else {
            ChoosePersonView.animateWithDurationAnimations(0.2, function () {
                _this.center = CGPointMake(_this.frontViewFrame.size.width / 2, -(_this.cardOffset * position) + (ratio * _this.cardOffset));
            });
        }
    };
    return ChoosePersonView;
}(MDCSwipeToChooseView));
//# sourceMappingURL=index.ios.js.map